@@ -19,12 +19,12 @@
 (defstruct (room-info (:constructor make-room-info (mask name kind))
                       (:copier nil))
     ;; the mask applied to HeaderValue to compute object size
-    (mask 0 :type (and fixnum unsigned-byte))
+    (mask 0 :type (and fixnum unsigned-byte) :read-only t)
     ;; the name of this type
     (name nil :type symbol :read-only t)
     ;; kind of type (how to reconstitute an object)
     (kind nil
-          :type (member :other :closure :instance :list :code :vector-nil)
+          :type (member :other :closure :instance :list :code)
           :read-only t))
 
 (defun room-info-type-name (info)
@@ -32,6 +32,7 @@
         (saetp-primitive-type-name info)
         (room-info-name info)))
 
+(defconstant tiny-boxed-size-mask #xFF)
 (defun !compute-room-infos ()
   (let ((infos (make-array 256 :initial-element nil))
         (default-size-mask (mask-field (byte 23 0) -1)))
@@ -43,16 +44,16 @@
                    (not (member widetag '(t nil))))
           (setf (svref infos (symbol-value widetag))
                 (make-room-info (if (member name '(fdefn symbol))
-                                    #xFF
+                                    tiny-boxed-size-mask
                                     default-size-mask)
                                 name :other)))))
 
-    (dolist (code (list #+sb-unicode complex-character-string-widetag
-                        complex-base-string-widetag simple-array-widetag
-                        complex-bit-vector-widetag complex-vector-widetag
-                        complex-array-widetag complex-vector-nil-widetag))
-      (setf (svref infos code)
-            (make-room-info default-size-mask 'array-header :other)))
+    (let ((info (make-room-info default-size-mask 'array-header :other)))
+      (dolist (code (list #+sb-unicode complex-character-string-widetag
+                          complex-base-string-widetag simple-array-widetag
+                          complex-bit-vector-widetag complex-vector-widetag
+                          complex-array-widetag complex-vector-nil-widetag))
+        (setf (svref infos code) info)))
 
     (setf (svref infos bignum-widetag)
           ;; Lose 1 more bit than n-widetag-bits because fullcgc robs 1 bit,
@@ -69,7 +70,7 @@
           (setf (svref infos (saetp-typecode saetp)) saetp))))
 
     (setf (svref infos simple-array-nil-widetag)
-          (make-room-info 0 'simple-array-nil :vector-nil))
+          (make-room-info 0 'simple-array-nil :other))
 
     (setf (svref infos code-header-widetag)
           (make-room-info 0 'code :code))
@@ -193,10 +194,6 @@
             (round-to-dualword (+ (* vector-data-offset n-word-bytes)
                                   n-data-octets)))))
 
-(defun code-component-size (x) ; in bytes
-  (declare (code-component x))
-  (round-to-dualword (+ (* (code-header-words x) n-word-bytes) (%code-code-size x))))
-
 (defun primitive-object-size (object)
   "Return number of bytes of heap or stack directly consumed by OBJECT"
   (if (is-lisp-pointer (get-lisp-obj-address object))
@@ -229,7 +226,7 @@
                                (nth-value 2 (reconstitute-vector
                                              object room-info))))))
                      (code-component
-                      (return-from primitive-object-size (code-component-size object)))
+                      (return-from primitive-object-size (code-object-size object)))
                      (t
                       ;; Other things (symbol, fdefn, value-cell, etc)
                       ;; don't have a sizer, so use GET-HEADER-DATA
@@ -275,31 +272,26 @@
                    list-pointer-lowtag
                    (* 2 n-word-bytes)))
 
-          (:closure ; also funcallable-instance
-           (values (tagged-object fun-pointer-lowtag)
-                   widetag
-                   (boxed-size (logand header-value short-header-max-words))))
-
           (:instance
            (values (tagged-object instance-pointer-lowtag)
                    widetag
                    (boxed-size (logand header-value short-header-max-words))))
 
-          (:other
-           (values (tagged-object other-pointer-lowtag)
+          (:closure ; also funcallable-instance
+           (values (tagged-object fun-pointer-lowtag)
                    widetag
-                   (boxed-size (logand header-value (room-info-mask info)))))
-
-          (:vector-nil
-           (values (tagged-object other-pointer-lowtag)
-                   simple-array-nil-widetag
-                   (* 2 n-word-bytes)))
+                   (boxed-size (logand header-value short-header-max-words))))
 
           (:code
            (let ((c (tagged-object other-pointer-lowtag)))
              (values c
                      code-header-widetag
-                     (code-component-size c))))))))))
+                     (code-object-size c))))
+
+          (:other
+           (values (tagged-object other-pointer-lowtag)
+                   widetag
+                   (boxed-size (logand header-value (room-info-mask info)))))))))))
 
 ;;; Iterate over all the objects in the contiguous block of memory
 ;;; with the low address at START and the high address just before
@@ -466,7 +458,7 @@ We could try a few things to mitigate this:
        ;; in fixedobj subspace, or code without enough header words
        ;; in varyobj subspace. (cf 'filler_obj_p' in gc-internal.h)
        (dx-flet ((filter (obj type size)
-                   (unless (consp obj)
+                   (unless (= type list-pointer-lowtag)
                      (funcall fun obj type size))))
          (map-immobile-objects #'filter :fixed))
        (dx-flet ((filter (obj type size)
@@ -544,7 +536,31 @@ We could try a few things to mitigate this:
                                     ,extra))))))
                  (map-objects-in-range fun start (range-end page-bytes-used)
                                        (< page-index initial-next-free-page))
-                 (setf start (range-end gencgc-card-bytes) span 0)))))))))
+                 ;; In a certain rare situation, we must restart the next loop iteration
+                 ;; at exactly PAGE-INDEX instead of 1+ PAGE-INDEX.
+                 ;; Normally the contents of PAGE-INDEX are always included in the
+                 ;; current range. But what if it contributed 0 bytes to the range?
+                 ;;
+                 ;;     N : #x8000 bytes used
+                 ;;   N+1 : #x8000 bytes used
+                 ;;   N+2 : 0 bytes used        <- PAGE-INDEX now points here
+                 ;;   N+3 : initially 0 bytes, then gets some bytes
+                 ;;
+                 ;; We invoked the map function with page_address(N) for #x10000 bytes.
+                 ;; Suppose that function consed a partly unboxed object starting on
+                 ;; page N+2 extending to page N+3, and that NEXT-FREE-PAGE was greater
+                 ;; than both to begin with, so the termination condition isn't in play.
+                 ;; In the best case scenario, resuming the scan at page N+3 (mid-object)
+                 ;; would read valid widetags, but in the worst case scenario, we get
+                 ;; random bits. Page N+2 needs a fighting chance to be the start of
+                 ;; a range, so go back 1 page if the current page had zero bytes used
+                 ;; and the span exceeded 1. That is, always make forward progress.
+                 (setq start (range-end (cond ((and (zerop page-bytes-used) (plusp span))
+                                               (decf page-index)
+                                               0)
+                                              (t
+                                               gencgc-card-bytes)))
+                       span 0)))))))))
 
   (do-rest-arg ((space) spaces)
     (if (eq space :dynamic)
@@ -584,10 +600,9 @@ We could try a few things to mitigate this:
              (hole-bytes 0))
     (map-immobile-objects
      (lambda (obj type size)
-       (declare (ignore type))
        (let ((address (logandc2 (get-lisp-obj-address obj) lowtag-mask)))
          (when (case subspace
-                 (:fixed (consp obj))
+                 (:fixed (= type list-pointer-lowtag))
                  (:variable (hole-p address)))
            (push (cons address size) holes)
            (incf hole-bytes size))))
@@ -621,8 +636,8 @@ We could try a few things to mitigate this:
 ;;; allocated in Space.
 (defun type-breakdown (space)
   (declare (muffle-conditions compiler-note))
-  (let ((sizes (make-array 256 :initial-element 0 :element-type '(unsigned-byte #.n-word-bits)))
-        (counts (make-array 256 :initial-element 0 :element-type '(unsigned-byte #.n-word-bits))))
+  (let ((sizes (make-array 256 :initial-element 0 :element-type 'word))
+        (counts (make-array 256 :initial-element 0 :element-type 'word)))
     (map-allocated-objects
      (lambda (obj type size)
        (declare (word size) (optimize (speed 3)) (ignore obj))
@@ -697,6 +712,12 @@ We could try a few things to mitigate this:
         (format t "~%Summary total:~%    ~:D bytes, ~:D objects.~%"
                 summary-total-bytes summary-total-objects)))))
 
+(declaim (ftype (sfunction (index &key (:comma-interval (and (integer 1) index))) index)
+                decimal-with-grouped-digits-width))
+(defun decimal-with-grouped-digits-width (value &key (comma-interval 3))
+  (let ((digits (length (write-to-string value :base 10))))
+    (+ digits (floor (1- digits) comma-interval))))
+
 ;;; Report object usage for a single space.
 (defun report-space-total (space-info cutoff)
   (declare (list space-info) (type (or single-float null) cutoff))
@@ -767,9 +788,13 @@ We could try a few things to mitigate this:
     (map-allocated-objects
      (lambda (obj type size)
        (declare (optimize (speed 3)))
-       (when (eql type instance-widetag)
+       (when (or (eql type instance-widetag)
+                 (eql type funcallable-instance-widetag))
          (incf total-objects)
-         (let* ((classoid (layout-classoid (%instance-layout obj)))
+         (let* ((layout (if (eql type funcallable-instance-widetag)
+                            (%funcallable-instance-layout obj)
+                            (%instance-layout obj)))
+                (classoid (layout-classoid layout))
                 (found (ensure-gethash classoid totals (cons 0 0)))
                 (size size))
            (declare (fixnum size))
@@ -817,6 +842,9 @@ We could try a few things to mitigate this:
 ;;; (2) sb-sprof deciding how many regions [sic] were made if #+cheneygc
 (defun get-page-size () sb-c:+backend-page-bytes+)
 
+;;; This function is sheer madness.  You're better off using
+;;; LIST-ALLOCATED-OBJECTS and then iterating over that, to avoid
+;;; seeing all the junk created while doing this thing.
 (defun print-allocated-objects (space &key (percent 0) (pages 5)
                                       type larger smaller count
                                       (stream *standard-output*))
@@ -913,24 +941,59 @@ We could try a few things to mitigate this:
 (defun list-allocated-objects (space &key type larger smaller count
                                      test)
   (declare (type (or (eql :all) spaces) space)
-           (type (or index null) larger smaller type count)
+           (type (or (unsigned-byte 8) null) type)
+           (type (or index null) larger smaller count)
            (type (or function null) test))
   (declare (dynamic-extent test))
-  (unless *ignore-after*
-    (setq *ignore-after* (cons 1 2)))
-  (collect ((counted 0 1+))
-    (let ((res ()))
-      (map-allocated-objects
-       (lambda (obj obj-type size)
-         (when (and (or (not type) (eql obj-type type))
-                    (or (not smaller) (<= size smaller))
-                    (or (not larger) (>= size larger))
-                    (or (not test) (funcall test obj)))
-           (setq res (maybe-cons space obj res))
-           (when (and count (>= (counted) count))
-             (return-from list-allocated-objects res))))
-       space)
-      res)))
+  (when (eql count 0)
+    (return-from list-allocated-objects nil))
+  ;; This function was pretty much random as to what subset of the heap it
+  ;; visited- it might see half the heap, 1/10th of the heap, who knows, because
+  ;; it stopped based on hitting a sentinel cons cell made just prior to the loop.
+  ;; That stopping condition was totally wrong because allocation does not occur
+  ;; linearly.  Taking 2 passes (first count, then store) stands a chance of
+  ;; getting a reasonable point-in-time view as long as other threads are not consing
+  ;; like crazy. If the user-supplied TEST function conses at all, then the result is
+  ;; still very arbitrary - including possible duplication of objects if we visit
+  ;; something and then see it again after GC transports it higher. The only way to
+  ;; allow consing in the predicate would be to use dedicated "arenas" for new
+  ;; allocations, that being a concept which we do not now - and may never - support.
+  (sb-int:dx-flet ((wantp (obj widetag size)
+                     (and (or (not type) (eql widetag type))
+                          (or (not smaller) (<= size smaller))
+                          (or (not larger) (>= size larger))
+                          (or (not test) (funcall test obj)))))
+    ;; Unless COUNT is smallish, always start by counting. Don't just trust the user
+    ;; because s/he might specify :COUNT huge-num which is acceptable provided that
+    ;; huge-num is an INDEX which could either exhaust the heap, or at least be
+    ;; wasteful if but a tiny handful of objects would actually satisfy WANTP.
+    (let* ((output (make-array
+                    (if (typep count '(integer 0 100000))
+                        count
+                        (let ((n 0))
+                          (map-allocated-objects
+                           (lambda (obj widetag size)
+                             (when (wantp obj widetag size) (incf n)))
+                           space)
+                          n))))
+           (index 0))
+      (block done
+       (map-allocated-objects
+        (lambda (obj widetag size)
+          (when (wantp obj widetag size)
+            (setf (aref output index) obj)
+            (when (= (incf index) (length output))
+              (return-from done))))
+        space))
+      (let ((list
+             (cond ((= index (length output)) ; easy case
+                    (coerce output 'list))
+                   (t ; didn't fill the array
+                    (collect ((res))
+                      (dotimes (i index (res))
+                        (res (svref output i))))))))
+        (fill output 0) ; assist GC a bit
+        list))))
 
 ;;; Calls FUNCTION with all objects that have (possibly conservative)
 ;;; references to them on current stack.
@@ -960,6 +1023,11 @@ We could try a few things to mitigate this:
     #+stack-grows-downward-not-upward (iter + *control-stack-end* sap>)
     #-stack-grows-downward-not-upward (iter - *control-stack-start* sap<)))
 
+(declaim (inline symbol-extra-slot-p))
+(defun symbol-extra-slot-p (x)
+  (> (logand (get-header-data x) tiny-boxed-size-mask)
+     (1- symbol-size)))
+
 ;;; Invoke FUNCTOID (a macro or function) on OBJ and any values in MORE.
 ;;; Note that neither OBJ nor items in MORE undergo ONCE-ONLY treatment.
 ;;; The fact that FUNCTOID can be a macro allows treatment of its first argument
@@ -1009,7 +1077,7 @@ We could try a few things to mitigate this:
                `(let ((.l. (%funcallable-instance-layout ,obj)))
                   ;; As for INSTANCE, allow the functoid to see the access form
                   (,functoid (%funcallable-instance-layout ,obj) ,@more)
-                  (,functoid (%funcallable-instance-function ,obj) ,@more)
+                  (,functoid (%funcallable-instance-fun ,obj) ,@more)
                   (ecase (layout-bitmap .l.)
                     (#.sb-kernel::+layout-all-tagged+
                      (loop for .i. from instance-data-start ; exclude layout
@@ -1039,11 +1107,13 @@ We could try a few things to mitigate this:
                `(%array-displaced-p ,obj)
                `(%array-displaced-from ,obj))
             ,.(make-case 'array)
-            ,.(make-case 'symbol
-               `(%primitive sb-c:fast-symbol-global-value ,obj)
-               `(symbol-info ,obj)
-               `(symbol-name ,obj)
-               `(symbol-package ,obj))
+            ,.(make-case* 'symbol
+               `(,functoid (%primitive sb-c:fast-symbol-global-value ,obj) ,@more)
+               `(,functoid (symbol-info ,obj) ,@more)
+               `(,functoid (symbol-name ,obj) ,@more)
+               `(,functoid (symbol-package ,obj) ,@more)
+               `(when (symbol-extra-slot-p ,obj)
+                  (,functoid (symbol-extra ,obj) ,@more)))
             ,.(make-case 'fdefn
                `(fdefn-name ,obj)
                `(fdefn-fun ,obj)
@@ -1062,8 +1132,9 @@ We could try a few things to mitigate this:
                ;; and/or visit the slots of each simple-fun but not the fun per se.
                )
             ,.(make-case '(or float (complex float) bignum
-                              #+sb-simd-pack simd-pack
-                              system-area-pointer)) ; nothing to do
+                           #+sb-simd-pack simd-pack
+                           #+sb-simd-pack-256 simd-pack-256
+                           system-area-pointer)) ; nothing to do
             ,.(make-case 'weak-pointer `(weak-pointer-value ,obj))
             ,.(make-case 'ratio `(%numerator ,obj) `(%denominator ,obj))
             ;; Visitor won't be invoked on (COMPLEX float)
@@ -1191,7 +1262,7 @@ We could try a few things to mitigate this:
   (values))
 
 #+nil ; for debugging
-(defun dump-dynamic-space-code (&optional (stream *standard-output*)
+(defun show-dynamic-space-code (&optional (stream *standard-output*)
                                 &aux (n-code-bytes 0)
                                      (total-pages next-free-page)
                                      (pages
@@ -1235,6 +1306,23 @@ We could try a few things to mitigate this:
               n-code-bytes n-pages waste
               (* 100 (/ waste tot))))))
 
+#+nil ; for debugging
+(defun show-immobile-spaces (which)
+  (flet ((show (obj type size)
+           (declare (ignore type size))
+           (let ((*print-pretty* nil))
+             (format t "~x: ~s~%" (get-lisp-obj-address obj) obj))))
+    (when (or (eq which :fixed) (eq which :both))
+      (format t "Fixedobj space~%==============~%")
+      (map-objects-in-range #'show
+        (%make-lisp-obj fixedobj-space-start)
+        (%make-lisp-obj (sap-int *fixedobj-space-free-pointer*))))
+    (when (or (eq which :variable) (eq which :both))
+      (format t "Varyobj space~%=============~%")
+      (map-objects-in-range #'show
+        (%make-lisp-obj varyobj-space-start)
+        (%make-lisp-obj (sap-int *varyobj-space-free-pointer*))))))
+
 #+gencgc
 (defun generation-of (object)
   (let* ((addr (get-lisp-obj-address object))
@@ -1247,6 +1335,19 @@ We could try a few things to mitigate this:
              (setq addr (get-lisp-obj-address (fun-code-header object))))
            (logand #xF (sap-ref-8 (int-sap (logandc2 addr lowtag-mask)) 3))))))
 
+;;; Show objects in a much simpler way than print-allocated-objects.
+;;; Probably don't use this for generation 0 of dynamic space. Other spaces are ok.
+;;; (And this is removed from the image; it's meant for developers)
+#+gencgc
+(defun show-generation-objs (gen space)
+  (let ((*print-pretty* nil))
+    (map-allocated-objects
+     (lambda (obj type size)
+       (declare (ignore type))
+       (when (= (generation-of obj) gen)
+         (format t "~x ~3x ~s~%" (get-lisp-obj-address obj) size obj)))
+     space)))
+
 ;;; Unfortunately this is a near total copy of the test in gc.impure.lisp
 (defun !ensure-genesis-code/data-separation ()
   #+gencgc
@@ -1287,6 +1388,52 @@ We could try a few things to mitigate this:
 ;;; this is a valid test that genesis separated code and data.
 (!ensure-genesis-code/data-separation)
 
+(defun hexdump (thing &optional (n-words nil wordsp)
+                                ;; pass NIL explicitly if T crashes on you
+                                (decode t))
+  (multiple-value-bind (obj addr count)
+      (if (integerp thing)
+          (values nil thing (if wordsp n-words 1))
+          (values
+           thing
+           (logandc2 (get-lisp-obj-address thing) lowtag-mask)
+           (if wordsp
+               n-words
+               (if (and (typep thing 'code-component) (plusp (code-n-entries thing)))
+                   ;; Display up through the first fun header
+                   (+ (code-header-words thing)
+                      (ash (%code-fun-offset thing 0) (- word-shift))
+                      simple-fun-code-offset)
+                   ;; at most 16 words
+                   (min 16 (ash (primitive-object-size thing) (- word-shift)))))))
+    (with-pinned-objects (obj)
+      (dotimes (i count)
+        (let ((word (sap-ref-word (int-sap addr) (ash i word-shift))))
+          (multiple-value-bind (lispobj ok) (if decode (make-lisp-obj word nil))
+            (let ((*print-lines* 1)
+                  (*print-pretty* t))
+              (format t "~x: ~v,'0x~:[~; = ~S~]~%"
+                      (+ addr (ash i word-shift))
+                      (* 2 n-word-bytes)
+                      word ok lispobj))))))))
+#+sb-thread
+(defun show-tls-map ()
+  (let ((list
+         (sort (sb-vm::list-allocated-objects
+                :all
+                :type sb-vm:symbol-widetag
+                :test (lambda (x) (plusp (sb-kernel:symbol-tls-index x))))
+               #'<
+               :key #'sb-kernel:symbol-tls-index))
+        (prev 0))
+    (dolist (x list)
+      (let ((n  (ash (sb-kernel:symbol-tls-index x) (- sb-vm:word-shift))))
+        (when (and (> n sb-thread::tls-index-start)
+                   (> n (1+ prev)))
+          (format t "(unused)~%"))
+        (format t "~5d = ~s~%" n x)
+        (setq prev n)))))
+
 (in-package "SB-C")
 ;;; As soon as practical in warm build it makes sense to add
 ;;; cold-allocation-point-fixups into the weak hash-table.
