@@ -1232,18 +1232,18 @@
 
 (defmethod compute-effective-slot-definition-initargs
     ((class slot-class) direct-slotds)
-  (let* ((name nil)
-         (initfunction nil)
-         (initform nil)
-         (initargs nil)
-         (allocation nil)
-         (allocation-class nil)
-         (type t)
-         (documentation nil)
-         (documentationp nil)
-         (namep  nil)
-         (initp  nil)
-         (allocp nil))
+  (let ((name nil)
+        (initfunction nil)
+        (initform nil)
+        (initargs nil)
+        (allocation nil)
+        (allocation-class nil)
+        (type t)
+        (documentation nil)
+        (documentationp nil)
+        (namep  nil)
+        (initp  nil)
+        (allocp nil))
 
     (dolist (slotd direct-slotds)
       (when slotd
@@ -1263,7 +1263,7 @@
           (setq allocation (slot-definition-allocation slotd)
                 allocation-class (slot-definition-class slotd)
                 allocp t))
-        (setq initargs (append (slot-definition-initargs slotd) initargs))
+        (setq initargs (union (slot-definition-initargs slotd) initargs))
         (let ((slotd-type (slot-definition-type slotd)))
           (setq type (cond
                        ((eq type t) slotd-type)
