@@ -46,7 +46,7 @@ set_pseudo_atomic_atomic(struct thread __attribute__((unused)) *thread)
 {
     lispobj *p = &THREAD_PA_BITS(thread);
     if (*p)
-        lose("set_pseudo_atomic_atomic: pseudo atomic bits is %d.", *p);
+        lose("set_pseudo_atomic_atomic: pseudo atomic bits is %"OBJ_FMTX, *p);
     __asm__ __volatile__
         ("or" LISPOBJ_ASM_SUFFIX " %0,%1"
          :
@@ -99,7 +99,7 @@ clear_pseudo_atomic_interrupted(struct thread *thread)
 
 #undef LISPOBJ_ASM_SUFFIX
 
-#elif (defined(LISP_FEATURE_ARM) || defined LISP_FEATURE_ARM64) && !defined LISP_FEATURE_SB_THREAD
+#elif (defined LISP_FEATURE_ARM || defined LISP_FEATURE_ARM64 || defined LISP_FEATURE_RISCV) && !defined LISP_FEATURE_SB_THREAD
 static inline int
 get_pseudo_atomic_atomic(struct thread *thread)
 {
@@ -140,7 +140,7 @@ clear_pseudo_atomic_interrupted(struct thread *thread)
 
 /* FIXME: Are these async signal safe? Compiler reordering? */
 
-#if !defined LISP_FEATURE_ARM && !defined LISP_FEATURE_ARM64
+#if !defined LISP_FEATURE_ARM && !defined LISP_FEATURE_ARM64 && !defined LISP_FEATURE_RISCV
 #define set_alloc_pointer(value) \
     (dynamic_space_free_pointer = \
      ((lispobj *) \
@@ -189,7 +189,7 @@ clear_pseudo_atomic_interrupted(struct thread *thread)
 
 #endif /* defined(LISP_FEATURE_GENCGC) */
 
-#if defined LISP_FEATURE_ARM || defined LISP_FEATURE_ARM64
+#if defined LISP_FEATURE_ARM || defined LISP_FEATURE_ARM64 || defined LISP_FEATURE_RISCV
 #define set_alloc_pointer(value)                \
     SetSymbolValue(ALLOCATION_POINTER, value, 0)
 #define get_alloc_pointer()                     \
